%Kim Ruhl, May 20, 2004
%This function solves the system of equations for the model.  There are T
%inratemperal FOCs and T-1 intertemperal FOCs, for a 2T-1 system of equations.
%The function uses Newton's method to solve the system of equations.
%The results are written to the file "output.txt"
%This function is called by depressions.m

function f = SolveModel(z)
f = 0;
global BETA;    global GAMMA;   global DELTA;   global ALPHA;   global G;
global K1;      global T;       global a;       global pop;     global lbar;
global tauc;     global taul;    global tauk;

ITR_MAX = 250;              %Maximum interations for Newton's method
EPSILON = 1.0e-06;          %Stopping value for Newton's method
J_STEP = 1.0e-05;          %Step size for computing numerical derivatives
N_STEP = 0.1;               %Begining step size in Newton's Method

x = zeros(2*T-1,1);             %variables to find
fx = zeros(2*T-1,1);            %value of the equations (what we want to be zero)
j = zeros(2*T-1,2*T-1);         %jacobian
j_inv = zeros(2*T-1,2*T-1);     %inverse of jacobian
xk = zeros(T+1,1);              %capital stock


%Use initial capital stock, and guess that capital grows at rate g
xk(1) = K1;
for i = 2:T
    xk(i) = G*xk(i-1);
end

%taking logs guarantees the variables will be positive
for i = 1:T-1
    x(i) = log(xk(i+1));
end

%guess that leisure is (1-GAMMA)*(1-taul(i)) of available time
for i = 1:T
    x(i+T-1) = log((1-GAMMA)*(1-taul(i))*lbar(i));
end

iteration = 0;
loss_fx = 1;
fx = EvalSystem(x);
%Newton's method
while (loss_fx >= EPSILON) & (iteration <= ITR_MAX)     %check: are we close enough to zero? have we tried too many times?
    j = Jacobian(x, J_STEP);
    j_inv = inv(j);
    fx = EvalSystem(x);
    if( iteration >= 20) N_STEP = 0.5; end      %Once things are going well, start taking bigger steps
    x = x - N_STEP*(j_inv*fx);                  %update guess     
    fx = EvalSystem(x);
    
    iteration = iteration + 1;
    loss_fx = sum(abs(fx));                     %find the error
    fprintf('iteration %5.2f \t',iteration);
    fprintf('error %10.6e \n', loss_fx);   
end

%check for failure
if iteration > ITR_MAX
    fprintf('Max number of iterations exceeded.');
    f = 1;
end

%since we logged everything, now we turn it back
xk(1) = K1;
for i = 2:T
    xk(i) = exp( x(i-1) );
end
xk(T+1) = xk(T)*G;  

%using the correct capital and leisure values, find output per worker,
%investment/GDP, hours worked per week, consumption/GDP, K/Y and the
%interest rate
results = zeros(T,6);
for i = 1:T
    y = a(i) * xk(i)^ALPHA * (lbar(i)-exp(x(i+T-1)))^(1-ALPHA);
    results(i,1) = y/1000/pop(i);
    results(i,2) = (xk(i+1) - (1-DELTA)*xk(i)) / y;
    results(i,3) = (lbar(i) - exp(x(i+T-1)))/lbar(i);
    results(i,4) = (y - xk(i+1) + (1-DELTA)*xk(i))/y;
    results(i,5) = xk(i)/y;
    results(i,6) = ALPHA * a(i) * xk(i)^(ALPHA-1) * (lbar(i)-exp(x(i+T-1)))^(1-ALPHA)-DELTA;
end

%write the data out to a file
save('output.txt','results','-ascii', '-tabs');
fprintf('Progam finished. Data written to file. \n');
%--------------------------------------------------------------------------
%----------------------------------Begin feval-----------------------------
%--------------------------------------------------------------------------
function f = EvalSystem(x)
%This function evaluates the system of equations.  It is called by Jacobian
%and SolveModel

%use the global parameter values
global BETA;    global GAMMA;   global DELTA;   global ALPHA;   global G;
global K1;      global T;       global a;       global pop;     global lbar;
global tauc;    global taul;    global tauk;

f = zeros(size(x,1),1);

xk = zeros(T+1,1);      %capital stocks
xl = zeros(T,1);        %leisure 
xn = zeros(T,1);        %labor
y = zeros(T,1);         %output
c = zeros(T,1);         %consumption
w = zeros(T,1);         %wages

%convert data from logs
xk(1) = K1;
for i = 2:T
    xk(i) = exp( x(i-1) );
end
xk(T+1) = G*xk(T);

for i = 1:T
    xl(i) = exp( x(i+T-1) );
end

%compute labor
for i = 1:T
     xn(i) = lbar(i) - xl(i);
 end
 
%compute output, consumption and wages
for i =1:T
   
    y(i) = a(i) * xk(i)^ALPHA * xn(i)^(1-ALPHA);
    c(i) = y(i) - xk(i+1) + (1-DELTA)*xk(i);
    w(i) = (1-ALPHA) * a(i) * xk(i)^ALPHA * xn(i)^-ALPHA;
end

%Compute the intratemperal FOC. The division by y(i)^0.4 is for scaling purposes. 
for i = 1:T
    f(i) = (w(i)*xl(i)*(1-taul(i)) - (1-GAMMA)/GAMMA*c(i)*(1+tauc(i)))/y(i)^0.4;
end

%Compute the intertemporal FOC.
for i = 1:T-1
    f(i+T) = c(i+1)/c(i)*(1+tauc(i+1))/(1+tauc(i)) - BETA*(1 + (1-tauk(i+1))*(ALPHA*a(i+1)*xk(i+1)^(ALPHA-1)*xn(i+1)^(1-ALPHA)-DELTA)) ; 
end
%--------------------------------------------------------------------------
%----------------------------------Begin Jacobian---------------------------
%--------------------------------------------------------------------------
function f = Jacobian(x, step)
% This function computes the Jacobian matrix of the function in feval at x. 
%The argument step is the size of the step used in numerical differentiation 
%-- usually a very small number.

%Create matrix of correct size to hold Jacobian and function values
n = size(x,1);
f = zeros(n,n);
fx = zeros(n,1);
fstep = zeros(n,1);

%For each function argument, increment the arguement by step, recompute
%the function value, f(x+step) and record [f(x)-f(x+step)]/step.  

fx = EvalSystem(x);
for c = 1:n
    x(c) = x(c) + step;
    xneg(c) = x(c) - step;
    fstep = EvalSystem(x);
    for r = 1:n
        f(r,c) = (fstep(r) - fx(r)) / step;
    end
    x(c) = x(c) - step;
end

